using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using TracPusher;
using CookComputing.XmlRpc;

namespace XmlRpc
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void button1_Click(object sender, EventArgs e)
        {
            Trac proxy;

            // Fill these in appropriately
            string user = "yourTracUserName";
            string password = "yourTracPassword";

            /// Create an instance of the Trac interface
            proxy = XmlRpcProxyGen.Create<Trac>();

            // If desired, point this to your URL. If you do not do this,
            // it will use the one specified in the service declaration.
            proxy.Url = "https://trac-rules.org/xmlrpc";

            // Attach your credentials
            proxy.Credentials = new System.Net.NetworkCredential(user, password);

            //PageAttributes attr;
            //attr.comment = "This is the comment that goes with the new page";
            //bool rc = proxy.putPage("SandBox", // new page name
            //           "''hi chris'', this page was automatically added via XmlRpc from .NET", // new page contents
            //           attr // new page attributes
            //);

            //List Wiki Pages OK
            //String[] pages = proxy.getAllPages();
            //foreach (String page in pages)
            //{
            //    Console.WriteLine("Page: " + page);
            //}

            Console.WriteLine();

            //List Methods
            String[] methods = proxy.SystemListMethods();
            foreach (String method in methods)
            {

                String[] comment = proxy.SystemMethodHelp(method).Split('\n');

                Console.WriteLine("/// <summary>");
                foreach (String s in comment)
                {
                    Console.WriteLine("/// " + s);
                }
                Console.WriteLine("/// </summary>");

                object[] sig = proxy.SystemMethodSignature(method);

                if (sig.Length > 0)
                {
                    //At Least a Return Type...
                    String[] parms = sig[0].ToString().Split(',');

                    //Normalize Parameters (array -> object[]
                    for (Int32 i = 0; i < parms.Length; i++)
                    {
                        if (parms[i] == "array")
                        {
                            parms[i] = "object[]";
                        }
                        if (parms[i] == "base64")
                        {
                            parms[i] = "string";
                        }                    
                        if (parms[i] == "boolean")
                        {
                            parms[i] = "bool";
                        }                                    
                        if (parms[i] == "dateTime.iso8601")
                        {
                            parms[i] = "string";
                        }
                        if (parms[i] == "struct")
                        {
                            parms[i] = "XmlRpcStruct";
                        }
                    }

                    //Build Parameter Help
                    for (Int32 i = 1; i < parms.Length; i++)
                    {
                        Console.WriteLine("/// <param name=\"p{0}\">{1}</param>", i, parms[i]);
                    }

                    //Build Return Type Help
                    Console.WriteLine("/// <returns>{0}</returns>", parms[0]);

                    //Build Parameter Signature
                    StringBuilder sb = new StringBuilder();
                    for (Int32 i = 1; i < parms.Length; i++)
                    {
                        sb.AppendFormat(String.Format("{0} p{1}\r\n", parms[i], i));
                    }

                    String signature = sb.ToString().Trim(new char[] { '\r', '\n' });
                    signature = signature.Replace('\r', ',');
                    signature = signature.Replace('\n', ' ');

                    //Write Method Declaration. 
                    Console.WriteLine("[XmlRpcMethod(\"{0}\")]", method);
                    Console.WriteLine("{0} {1}({2});", parms[0], method.Replace('.', '_'), signature);
                }
                else
                {
                    //Write Method Declaration. 
                    Console.WriteLine("[XmlRpcMethod(\"{0}\")]", method);
                    Console.WriteLine("{0} {1}({2});", "void", method.Replace('.', '_'), "");
                }
                Console.WriteLine();
            }
        }
    }
}
